# -*- coding: utf-8 -*-
# This file is part of SWG (Static Website Generator).
#
# Copyright(c) 2010-2011 Simone Margaritelli
# evilsocket@gmail.com
# http://www.evilsocket.net
# http://www.backbox.org
#
# This file may be licensed under the terms of of the
# GNU General Public License Version 2 (the ``GPL'').
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the GPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the GPL along with this
# program. If not, go to http://www.gnu.org/licenses/gpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
import os
import codecs

from swg.core.config import Config

class DiffManager:
  __instance = None

  def __init__(self):
    self.changes  = {}
    self.sitename = Config.getInstance().sitename
    self.home     = os.path.expanduser( "~/.swg/" )
    self.sitebase = self.home + self.sitename + os.sep

    if not os.path.exists(self.home):
      os.mkdir(self.home)

    if not os.path.exists(self.sitebase):
      os.mkdir(self.sitebase)


  def __save_digest( self, digest_path, digest ):
    fd = codecs.open( digest_path, "w+", "utf-8" )
    fd.write( digest.strip() )
    fd.close()

  def checkItem( self, filename, hash_id, digest ):
    # a new file
    if not os.path.exists( self.sitebase + hash_id ):
      self.changes[filename] = ( digest, 'NEW' )
      self.__save_digest( self.sitebase + hash_id, digest )
    # existing file, check digests
    else:
      fd = codecs.open( self.sitebase + hash_id, "r", "utf-8" )

      old_digest = fd.read().strip()
      # changes!
      if digest != old_digest:
        self.changes[filename] = ( digest, 'MODIFIED' )
        self.__save_digest( self.sitebase + hash_id, digest )
          
      fd.close()

  @classmethod
  def getInstance(cls):
    if cls.__instance is None:
      cls.__instance = DiffManager()
    return cls.__instance



# -*- coding: utf-8 -*-
# This file is part of SWG (Static Website Generator).
#
# Copyright(c) 2010-2011 Simone Margaritelli
# evilsocket@gmail.com
# http://www.evilsocket.net
# http://www.backbox.org
#
# This file may be licensed under the terms of of the
# GNU General Public License Version 2 (the ``GPL'').
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the GPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the GPL along with this
# program. If not, go to http://www.gnu.org/licenses/gpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
import re
import os
import hashlib

from swg.core.config      import Config
from swg.core.diffmanager import DiffManager
from swg.core.pager       import Pager

class Item:
  SLUGIFY_SPLIT_REGEXP  = re.compile( r'[^\w]+' )
  PAGER_ENABLED_CLASSES = (
    'swg.entities.category.Category',
    'swg.entities.tag.Tag',
    'swg.entities.author.Author'
  )

  def __init__( self, path, title, extension ):
    self.path        = path.replace( '//', '/' ) 
    self.title       = title
    self.extension   = extension
    self.name        = self.__generate_name()
    self.url         = ("%s/%s.%s" % (self.path,self.name,self.extension)).replace( '//', '/' )
    self.objects     = {}
    self.hash_id     = hashlib.md5( self.url.encode( "utf-8" ) ).hexdigest()
    self.digest      = ""
    self.npages      = 1
    self.gzip        = Config.getInstance().gzip
    self.compression = Config.getInstance().compression
    self.gzip_allow  = re.compile( '^.+\.' + Config.getInstance().page_ext + '$', re.IGNORECASE )

  def __generate_name( self ):
      result = []
      for word in Item.SLUGIFY_SPLIT_REGEXP.split( self.title.lower() ):
          result.extend( word.split() )

      return '-'.join(result)

  def __save_contents( self, filename, contents ):
    if self.gzip is True and self.gzip_allow.match( filename ):
      import gzip
      import cStringIO

      fdio = cStringIO.StringIO()
      fd   = gzip.GzipFile( mode = 'wb',  fileobj = fdio, compresslevel = 9 )
      fd.write( contents )
      fd.close()

      filename = filename + u'.gz'
      contents = fdio.getvalue()

    fd = open( filename.encode('UTF-8'), "w+b" )
    fd.write( contents )
    fd.close()
  
  def addObject( self, name, value ):
    self.objects[name] = value
    
    if hasattr( self, 'author') and self.author is not None:
      self.author.addObject( name, value )
    
    if hasattr( self, 'categories' ):
      for category in self.categories:
        category.addObject( name, value )
    
    if hasattr( self, 'tags' ):
      for tag in self.tags:
        tag.addObject( name, value )

    return self

  def create(self):  
    config = Config.getInstance()
    path   = config.outputpath + os.sep + self.path

    if not os.path.exists( path ):
      os.mkdir(path)

    if config.pager == True and (self.title == 'index' or str(self.__class__) in Item.PAGER_ENABLED_CLASSES):     
      pager = Pager( '%s.%s'     % ( self.name, self.extension ),
                     '%s-%%d.%s' % ( self.name, self.extension ) )
      
      if hasattr( self, 'items' ):
        pager.setPages( self.items )
      else:
        pager.setPages( self.objects['pages'] )

      self.npages = pager.getTotalPages()

      self.addObject( 'pager', pager )

      for filename in pager:
        filename = os.path.join( path, filename )
        content  = self.render()

        self.digest = hashlib.md5( content ).hexdigest()

        DiffManager.getInstance().checkItem( filename, self.hash_id, self.digest )

        self.__save_contents( filename, content )
    else:
      filename = os.path.join( path, "%s.%s" % (self.name, self.extension) )
      content  = self.render()

      self.digest = hashlib.md5( content ).hexdigest()

      DiffManager.getInstance().checkItem( filename, self.hash_id, self.digest )

      self.__save_contents( filename, content )



# -*- coding: utf-8 -*-
# This file is part of SWG (Static Website Generator).
#
# Copyright(c) 2010-2011 Simone Margaritelli
# evilsocket@gmail.com
# http://www.evilsocket.net
# http://www.backbox.org
#
# This file may be licensed under the terms of of the
# GNU General Public License Version 2 (the ``GPL'').
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the GPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the GPL along with this
# program. If not, go to http://www.gnu.org/licenses/gpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
from swg.entities.page        import Page
from swg.core.itemparser      import ItemParser
from swg.core.authormanager   import AuthorManager
from swg.core.categorymanager import CategoryManager
from swg.core.tagmanager      import TagManager

class PageParser(ItemParser):
  MANDATORY_FIELDS = {
    'date'       : 'datetime',
    'author'     : 'string',
    'categories' : 'array',
    'tags'       : 'array',
    'title'      : 'string'
  }
  
  def __init__(self):
    ItemParser.__init__(self)
        
  def parse( self, filename ):
    ItemParser.parse( self, PageParser.MANDATORY_FIELDS, filename )

    page = Page( self.info['title'] )

    author = AuthorManager.getInstance().get( self.info['author'] )
    author.items.append(page)
    
    categories = []
    for title in self.info['categories']:
      category = CategoryManager.getInstance().get(title)
      category.items.append(page)
      categories.append( category )

    tags = []
    for title in self.info['tags']:
      tag = TagManager.getInstance().get(title)
      tag.items.append(page)
      tags.append( tag )

    page.datetime   = self.info['date']
    page.author     = author
    page.categories = categories
    page.tags       = tags
    page.abstract   = self.abstract
    page.content    = self.body

    # reset the state
    ItemParser.__init__(self)

    return page



# -*- coding: utf-8 -*-
# This file is part of SWG (Static Website Generator).
#
# Copyright(c) 2010-2011 Simone Margaritelli
# evilsocket@gmail.com
# http://www.evilsocket.net
# http://www.backbox.org
#
# This file may be licensed under the terms of of the
# GNU General Public License Version 2 (the ``GPL'').
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the GPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the GPL along with this
# program. If not, go to http://www.gnu.org/licenses/gpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
import os

from swg.entities.item        import Item
from swg.core.config          import Config
from swg.core.templatemanager import TemplateManager

class Author(Item):
  def __init__( self, username ):
    Item.__init__( self, Config.getInstance().basepath + os.sep + 'members', username, Config.getInstance().page_ext )
    self.username = username
    self.items    = []
    self.avatar   = ""
    self.email    = ""
    self.website  = ""
    self.abstract = ""
    self.content  = ""
    self.template = TemplateManager.getInstance().get('author.tpl')
    self.sorted   = False

  def render( self ):
    if not self.sorted:
      self.items.sort( reverse=True, key=lambda item: item.datetime )
      self.sorted = True

    return TemplateManager.render( template = self.template, author = self, **self.objects )




# -*- coding: utf-8 -*-
# This file is part of SWG (Static Website Generator).
#
# Copyright(c) 2010-2011 Simone Margaritelli
# evilsocket@gmail.com
# http://www.evilsocket.net
# http://www.backbox.org
#
# This file may be licensed under the terms of of the
# GNU General Public License Version 2 (the ``GPL'').
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the GPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the GPL along with this
# program. If not, go to http://www.gnu.org/licenses/gpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
from swg.core.itemparser import ItemParser
from swg.entities.author import Author

class AuthorParser(ItemParser):
  MANDATORY_FIELDS = {
    'username' : 'string',
    'avatar'   : 'string',
    'email'    : 'string',
    'website'  : 'string'
  }

  def __init__(self):
    ItemParser.__init__(self)

  def parse( self, filename ):
    ItemParser.parse( self, AuthorParser.MANDATORY_FIELDS, filename )
    
    object = Author( self.info['username'] )
    object.avatar   = self.info['avatar']
    object.email    = self.info['email']
    object.website  = self.info['website']
    object.content  = self.body
    object.abstract = self.abstract

    return object



# -*- coding: utf-8 -*-
# This file is part of SWG (Static Website Generator).
#
# Copyright(c) 2010-2011 Simone Margaritelli
# evilsocket@gmail.com
# http://www.evilsocket.net
# http://www.backbox.org
#
# This file may be licensed under the terms of of the
# GNU General Public License Version 2 (the ``GPL'').
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the GPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the GPL along with this
# program. If not, go to http://www.gnu.org/licenses/gpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
import codecs
import datetime 

from swg.core.config import Config

class ItemParser:
  PARSE_NONE_STATE = 0
  PARSE_INFO_STATE = 1
  PARSE_BODY_STATE = 2
  PARSE_DONE_STATE = 3

  BODY_ABSTRACT_BREAK = u'<break>'

  TIDY_OPTIONS = {  'alt-text'          : ' ',
                    'doctype'           : 'transitional',
                    'bare'              : 1,
                    'clean'             : 0,
                    'hide-comments'     : 1,
                    'join-classes'      : 1,
                    'join-styles'       : 1,
                    'output-xhtml'      : 1,
                    'quote-nbsp'        : 0,
                    'preserve-entities' : 1,
                    'show-errors'       : 0,
                    'show-warnings'     : 0,
                    'wrap'              : 0,
                    'sort-attributes'   : 'alpha',
                    'char-encoding'     : 'utf8',
                    'input-encoding'    : 'utf8',
                    'output-encoding'   : 'utf8',
                    'indent'            : 0,
                    'indent-spaces'     : 0,
                    'newline'           : 'LF',
                    'output-bom'        : 0,
                    'force-output'      : 1,
                    'quiet'             : 1,
                    'tidy-mark'         : 0,
                    'show-body-only'    : 1
                  }

  def __init__(self):
    self.info     = {}
    self.abstract = ""
    self.body     = ""
    self.state    = ItemParser.PARSE_NONE_STATE
    self.lineno   = 1

  def __parse_datetime( self, data ):
    return datetime.datetime.strptime( data, '%Y-%m-%d %H:%M:%S' )

  def __parse_string( self, data ):
    return data.strip()

  def __parse_array( self, data ):
    data  = data.strip()
    items = data.split(',')
    return map( lambda s: s.strip(), items )

  def parse( self, mandatory_fields_map, filename ):
    fd = codecs.open( filename, "r", "utf-8" )

    self.state = ItemParser.PARSE_INFO_STATE

    for line in iter(fd):
      if self.state == ItemParser.PARSE_INFO_STATE:
        line = line.strip()
        if line == '':
          self.state = ItemParser.PARSE_BODY_STATE
        else:
          ( info_id, info_data ) = line.split( ':', 1 )
          info_id   = info_id.strip().lower()
          info_data = info_data.strip()

          if mandatory_fields_map.has_key(info_id):
            type = mandatory_fields_map[info_id]
            if type == 'datetime':
              self.info[info_id] = self.__parse_datetime(info_data)
            elif type == 'string':
              self.info[info_id] = self.__parse_string(info_data)
            elif type == 'array':
              self.info[info_id] = self.__parse_array(info_data)

          else:
            raise Exception( "Unknown key %s on line %d." % ( info_id, self.lineno ) )

      elif self.state == ItemParser.PARSE_BODY_STATE:
        self.body += line
      else:
        raise Exception( "Unhandled parser state on line %d." % self.lineno )

      self.lineno += 1

    missing = filter( lambda x:x not in self.info.keys(), mandatory_fields_map.keys() )
    if missing != []:
      raise Exception( "Missing mandatory fields : %s" % ', '.join(missing) )

    if ItemParser.BODY_ABSTRACT_BREAK in self.body:
      ( self.abstract, therest ) = self.body.split( ItemParser.BODY_ABSTRACT_BREAK, 1 )
      self.body = self.abstract.strip() + '<br/><br/>' + therest.strip()
    else:
      self.abstract = self.body

    # Fix <break> pseudo attribute newlines
    self.body     = self.body.replace( "\n\n", "<br/><br/>" )
    self.abstract = self.abstract.replace( "\n\n", "<br/><br/>" )

    # Tidyfy abstract and body html and fix encoding errors
    if Config.getInstance().tidyfy is True:
      import tidy

      self.body     = tidy.parseString( self.body.encode( 'UTF-8',     'replace' ),     **ItemParser.TIDY_OPTIONS )
      self.abstract = tidy.parseString( self.abstract.encode( 'UTF-8', 'replace' ), **ItemParser.TIDY_OPTIONS )
      self.body     = str(self.body).decode('UTF-8')
      self.abstract = str(self.abstract).decode('UTF-8')

    fd.close()



# -*- coding: utf-8 -*-
# This file is part of SWG (Static Website Generator).
#
# Copyright(c) 2010-2011 Simone Margaritelli
# evilsocket@gmail.com
# http://www.evilsocket.net
# http://www.backbox.org
#
# This file may be licensed under the terms of of the
# GNU General Public License Version 2 (the ``GPL'').
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the GPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the GPL along with this
# program. If not, go to http://www.gnu.org/licenses/gpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
import os

from swg.entities.item        import Item
from swg.core.config          import Config
from swg.core.templatemanager import TemplateManager

class Page(Item):
  def __init__( self, title, template = None ):
    Item.__init__( self, Config.getInstance().basepath, title, Config.getInstance().page_ext )
    self.datetime   = None
    self.author     = None
    self.categories = []
    self.tags       = []
    self.abstract   = ""
    self.content    = ""
    self.template   = TemplateManager.getInstance().get( 'page.tpl' if template is None else template )

  def render( self ):
    return TemplateManager.render( template = self.template, page = self, **self.objects )
    
  def create( self ):
    Item.create(self)

    # create only authors not already done
    if self.author != None and not os.path.exists( os.path.join( Config.getInstance().outputpath, self.author.url ) ):
      self.author.create()

    for category in self.categories:
      # create only categories not already done
      if not os.path.exists( os.path.join( Config.getInstance().outputpath, category.url ) ):
        category.create()

    for tag in self.tags:
      # create only tags not already done
      if not os.path.exists( os.path.join( Config.getInstance().outputpath, tag.url ) ):
        tag.create()



# -*- coding: utf-8 -*-
# This file is part of SWG (Static Website Generator).
#
# Copyright(c) 2010-2011 Simone Margaritelli
# evilsocket@gmail.com
# http://www.evilsocket.net
# http://www.backbox.org
#
# This file may be licensed under the terms of of the
# GNU General Public License Version 2 (the ``GPL'').
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the GPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the GPL along with this
# program. If not, go to http://www.gnu.org/licenses/gpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
import codecs

from swg.entities.category import Category
from swg.core.config       import Config

class CategoryManager:
  __instance = None

  def __init__(self):
    self.categories = {}
    self.hierarchy  = []

  def get( self, title = None ):
    if title != None:
      id = title.lower()
      if not self.categories.has_key(id):
        self.categories[id] = Category(title)

      return self.categories[id]
    else:
      if self.hierarchy == []:
        self.__build_hierarchy()

      return self.hierarchy

  def __find_category( self, title ):
    for category in self.categories.values():
      if category.title == title:
        return category
    return None

  def __find_in_hyerarchy( self, title, node ):
    if node.title == title:
      return node
      
    for child in node.children:
      if child.title == title:
        return child
      else:
        found = self.__find_in_hyerarchy( title, child )
        if found != None:
          return found

    return None

  # only for debug purpose
  def __print_hierarchy(self,node,tabs = 0):
    print "%s%s :" % ( "\t" * tabs, node.title )
    for child in node.children:
      self.__print_hierarchy( child, tabs + 1 )

  def __build_hierarchy(self):
    fd = codecs.open( Config.getInstance().hierarchy, "r", "utf-8" )

    for line in iter(fd):
      line = line.strip()
      if line != '':
        if ':' in line:
          ( root, children ) = line.split( ':', 1 )
          root     = root.strip()
          children = children.strip().split(',')
          children = map( lambda s: s.strip(), children )
        else:
          root     = line
          children = []

        h_root = None
        
        for category in self.hierarchy:
          h_root = self.__find_in_hyerarchy( root, category )
          if h_root != None:
            break

        if h_root == None:
          h_root = self.__find_category(root)
          if h_root != None:
            self.hierarchy.append(h_root)
       
        for child in children:
          h_child = self.__find_category(child)
          if h_root is not None and h_child is not None:
            h_root.children.append(h_child)

    fd.close()

  @classmethod
  def getInstance(cls):
    if cls.__instance is None:
      cls.__instance = CategoryManager()
    return cls.__instance



# -*- coding: utf-8 -*-
# This file is part of SWG (Static Website Generator).
#
# Copyright(c) 2010-2011 Simone Margaritelli
# evilsocket@gmail.com
# http://www.evilsocket.net
# http://www.backbox.org
#
# This file may be licensed under the terms of of the
# GNU General Public License Version 2 (the ``GPL'').
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the GPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the GPL along with this
# program. If not, go to http://www.gnu.org/licenses/gpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
from swg.entities.tag import Tag

class TagManager:
  __instance = None

  def __init__(self):
    self.tags   = {}
    self.sorted = None

  def get( self, title = None ):
    if title != None:
      id = title.lower()
      if not self.tags.has_key(id):
        self.tags[id] = Tag(title)

      return self.tags[id]
    else:
      if self.sorted is None:
        self.sorted = self.tags.values()
        self.sorted.sort( reverse=True, key=lambda t: len(t.items) )

      return self.sorted

  @classmethod
  def getInstance(cls):
    if cls.__instance is None:
      cls.__instance = TagManager()
    return cls.__instance



# -*- coding: utf-8 -*-
# This file is part of SWG (Static Website Generator).
#
# Copyright(c) 2010-2011 Simone Margaritelli
# evilsocket@gmail.com
# http://www.evilsocket.net
# http://www.backbox.org
#
# This file may be licensed under the terms of of the
# GNU General Public License Version 2 (the ``GPL'').
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the GPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the GPL along with this
# program. If not, go to http://www.gnu.org/licenses/gpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

import os
import sys
import os.path
import re
import shutil
import SimpleHTTPServer
import SocketServer

from swg.core.config      import Config
from swg.core.pageparser  import PageParser
from swg.entities.page    import Page
from swg.core.diffmanager import DiffManager

class Engine:

  def __init__(self):
    self.config = Config.getInstance()
    self.dbdir  = os.path.join( self.config.dbpath, 'pages' )
    self.files  = os.listdir( self.dbdir ) if os.path.exists( self.dbdir ) else []
    self.path   = os.path.dirname( os.path.realpath( __file__ ) )

  def new( self ):
    maxid = 0
    for file in self.files:
      ( id, ext ) = file.split('.')
      if int(id) > maxid:
        maxid = int(id)
    
    newitem = os.path.join( self.dbdir, "%d.%s" % (maxid + 1, self.config.dbitem_ext) )
    fd      = open( newitem, 'w+t' )

    fd.write( """\
Date: %s
Author:
Categories:
Tags:
Title:

""" % self.config.now.strftime("%Y-%m-%d %H:%M:%S") )

    fd.close()
    
    os.system( "%s %s" % ( self.config.editor, newitem ) )

    if os.path.exists( newitem ):
      print "@ Item '%s' created, you can now regenerate the website ." % newitem
    else:
      print "@ Item was not saved, quitting ."

  def create( self, destfolder ):
    if os.path.exists(destfolder):
      sys.exit( "@ The folder '%s' already exists, operation interrupted for security reasons." % destfolder )  
    else:
      print "@ Creating SWG basic website structure inside the '%s' folder ..." % destfolder

      shutil.copytree( os.path.join( self.path, 'basic' ), destfolder )

      print """\
@ Basic website initialized, now run:

  cd %s
  swg --generate

To generate the html contents or:

  cd %s
  swg --serve

To test the website locally.""" % (destfolder,destfolder)

  def serve( self ):

    class SWGServer(SocketServer.TCPServer):
      allow_reuse_address = True

    self.config.siteurl = 'http://localhost:8080'
    self.generate()

    os.chdir( self.config.outputpath )

    print "\n@ Serving the site on http://localhost:8080/ press ctrl+c to exit ..."
    
    try:
      SWGServer( ("",8080), SimpleHTTPServer.SimpleHTTPRequestHandler ).serve_forever()
    except KeyboardInterrupt:
      print "\n@ Bye :)"

  def generate( self ):
    parser = PageParser( )
    pages  = []

    print "@ Parsing pages ..."
    for file in self.files:
      if re.match( '^.+\.' + self.config.dbitem_ext + '$', file ):
        filename = os.path.join( self.dbdir, file )
        page     = parser.parse( filename )
        pages.append(page)

    print "@ Sorting pages by date ..."
    pages.sort( reverse=True, key=lambda p: p.datetime )

    # delete output directory and recreate it
    if os.path.exists(self.config.outputpath ):
      print "@ Removing old '%s' path ..."  % self.config.outputpath
      shutil.rmtree( self.config.outputpath )

    print "@ Creating '%s' path ..." % self.config.outputpath
    os.mkdir( self.config.outputpath )

    for source, destination in self.config.copypaths.items():
      print "@ Importing '%s' to '%s' ..." % (source, destination)
      if os.path.isfile(source):
        shutil.copy( source, destination )
      elif os.path.isdir(source):
        shutil.copytree( source, destination )
      else:
        raise Exception("Unexpected type of '%s' ." % source )

    if os.path.exists( os.path.join( self.config.tplpath, 'index.tpl' ) ):
      print "@ Creating index file ..."
      index = Page( 'index', 'index.tpl' ).addObject( 'pages', pages )
      index.create()
    else:
      raise Exception( "No index template found." )

    if os.path.exists( os.path.join( self.config.tplpath, '404.tpl' ) ):
      print "@ Creating 404 file ..."
      Page( '404', '404.tpl' ).addObject( 'pages', pages ).create()

    if os.path.exists( os.path.join( self.config.tplpath, 'feed.tpl' ) ):
      print "@ Creating feed.xml file ..."
      feed = Page( 'feed', 'feed.tpl' )
      feed.addObject( 'pages', pages )
      feed.extension = 'xml'
      feed.create()

    print "@ Rendering %d pages ..." % len(pages)
    for page in pages:
      page.addObject( 'pages', pages ).create()

    if os.path.exists( os.path.join( self.config.tplpath, 'sitemap.tpl' ) ):
      print "@ Creating sitemap.xml file ..."
      sitemap = Page( 'sitemap', 'sitemap.tpl' )
      sitemap.addObject( 'index', index )
      sitemap.addObject( 'pages', pages )
      sitemap.extension = 'xml'
      sitemap.create()

    if self.config.gzip is True:
      htaccess = os.path.join( self.config.outputpath, '.htaccess' ) 
      if os.path.exists( htaccess ):
        fd = open( htaccess, "a+t" )
        fd.write( """
  # SWG Generated Code
  AddEncoding gzip .gz
  DirectoryIndex index.html index.htm index.shtml index.php index.php4 index.php3 index.phtml index.cgi index.html.gz

  <Files *.""" + self.config.page_ext + """.gz>
    ForceType text/html
  </Files>

  <FilesMatch .*\.(""" + self.config.page_ext + """)>
    RewriteEngine on
    RewriteCond %{REQUEST_FILENAME}.gz -f
    RewriteRule ^(.*)$ $1.gz [L]
  </FilesMatch>""" )
        fd.close()
      else:
        fd = open( htaccess, "w+t" )
        fd.write( """
  # SWG Generated Code
  Options +FollowSymlinks
  RewriteEngine on

  AddEncoding gzip .gz
  DirectoryIndex index.html index.htm index.shtml index.php index.php4 index.php3 index.phtml index.cgi index.html.gz

  <Files *.""" + self.config.page_ext + """.gz>
    ForceType text/html
  </Files>

  <FilesMatch .*\.(""" + self.config.page_ext + """)>
    RewriteEngine on
    RewriteCond %{REQUEST_FILENAME}.gz -f
    RewriteRule ^(.*)$ $1.gz [L]
  </FilesMatch>""" )
        fd.close()

    print "@ DONE\n"

    if self.config.transfer is not None:
      os.system( self.config.transfer.encode( "UTF-8" ) )
    else:
      for filename, info in DiffManager.getInstance().changes.items():
        ( digest, status ) = info
        print "@ %-8s : '%s'" % ( status, filename.encode( "UTF-8" ) )




#!/usr/bin/python 
# -*- coding: utf-8 -*-
# This file is part of SWG (Static Website Generator).
#
# Copyright(c) 2010-2011 Simone Margaritelli
# evilsocket@gmail.com
# http://www.evilsocket.net
# http://www.backbox.org
#
# This file may be licensed under the terms of of the
# GNU General Public License Version 2 (the ``GPL'').
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the GPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the GPL along with this
# program. If not, go to http://www.gnu.org/licenses/gpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
import os
import re
import sys
import urllib
import codecs
from xml.dom  import minidom
from datetime import datetime
from optparse import OptionParser

def slugify_name( name ):
  result = []
  for word in slugify.split( name.lower() ):
    result.extend( word.split() )

  return '-'.join(result)

print "- SWG Wordpress Backup Importer -\n"

parser = OptionParser( usage = "usage: %prog -i wordpress-backup.xml -u 'http://www.your-site-url.com' <options>\n" )

parser.add_option( "-i", "--input",	action="store", dest="wpbackup", default=None,  help="The Wordpress XML backup file." )
parser.add_option( "-u", "--url", 	action="store", dest="siteurl",  default=None,  help="URL of the destination website." )
parser.add_option( "-o", "--output",	action="store", dest="outdir",   default=".",   help="Output directory, default is the current working directory." )
parser.add_option( "-e", "--extension", action="store", dest="fileext",  default="txt", help="Output file extension, default is txt." )
parser.add_option( "-I", "--images",    action="store", dest="imgdir",   default=None,  help="If specified, it's the path where the importer will try to download images referenced by articles." )

(o,args) = parser.parse_args()

if o.wpbackup is None:
  parser.error( "Wordpress XML backup file not specified !")
elif o.siteurl is None:
  parser.error( "URL of the destination website not specified !" )

wpbackup    = o.wpbackup 
siteurl     = o.siteurl
fileext     = o.fileext
outdir      = o.outdir
imgdir      = o.imgdir
imgdownload = True if imgdir is not None else False

print "@ Loading %s ..." % wpbackup

doc     = minidom.parse(wpbackup)
items   = doc.getElementsByTagName('item')
domain  = re.sub( '^(https?\://)?(www\.)?', '', siteurl )
imgtag  = re.compile( '<\s*img[^/>]+src\s*=\s*[\'"]([^\'"]+' + re.escape(domain) + '[^\'"]+)[\'"]',re.IGNORECASE )
slugify = re.compile( r'[^\w]+' )

post_id         = 0
site_authors    = []
site_categories = []

# create needed directories
if not os.path.exists(outdir):
  os.mkdir( outdir )       

try:
  os.makedirs( outdir + os.sep + 'db' + os.sep + 'pages' )
except:
  pass

if imgdownload is True and not os.path.exists(imgdir):
  os.mkdir( imgdir )

for item in items:
  status = item.getElementsByTagName('wp:status')[0].firstChild.nodeValue
  type   = item.getElementsByTagName('wp:post_type')[0].firstChild.nodeValue
  # import only published posts
  if status == 'publish' and type == 'post':
    title      = item.getElementsByTagName('title')[0].firstChild.nodeValue
    date       = item.getElementsByTagName('wp:post_date')[0].firstChild.nodeValue
    author     = item.getElementsByTagName('dc:creator')[0].firstChild.nodeValue
    content    = item.getElementsByTagName('content:encoded')[0].firstChild.nodeValue
    categories = []
    tags       = []

    print "@ Processing '%s' ..." % title

    # get item categories and tags
    metas = item.getElementsByTagName('category')
    for meta in metas:
      domain = meta.attributes['domain'].value if meta.attributes.has_key('domain') else 'category'
      value  = meta.firstChild.nodeValue.strip()
      
      if domain == 'category' and value not in categories:
        categories.append(value)
      elif domain == 'tag' and value not in tags:
        tags.append(value)
        
    # replace the 'more' pseudo tag in the content with swg <break> 
    content = content.replace( '<!--more-->', '<break>' )
    # make sure date is correct attempting to parse it
    date    = datetime.strptime( date, '%Y-%m-%d %H:%M:%S' )
    # search for images to download
    if imgdownload is True:
      images = imgtag.findall(content)
      for image in images:
        # update the html with the new image url and download it to the specified directory
        imgname = os.path.basename(image)
        content = content.replace( image, siteurl + '/' + os.path.basename(imgdir) + '/' + imgname )
        imgfile = imgdir + os.sep + imgname 
        if not os.path.exists(imgfile):
          print "\t- downloading '%s' to %s ..." % ( image, imgfile )
          urllib.urlretrieve( image, imgfile )

    data = """\
Date: %s
Author: %s
Categories: %s
Tags: %s
Title: %s\n\n""" % ( date.strftime('%Y-%m-%d %H:%M:%S'), author, ', '.join(categories), ', '.join(tags), title ) + content
  
    filename = outdir + os.sep + 'db' + os.sep + 'pages' + os.sep + "%d.%s" % ( post_id, fileext )
    file     = codecs.open( filename, "w+", "utf-8")
    file.write(data)
    file.close()

    print "@ Saved to %s ." % filename

    # save author if not already present in the list
    if author not in site_authors:
      site_authors.append(author)
    # same for the categories
    for category in categories:
      if category not in site_categories:
        site_categories.append(category)

    post_id += 1
   
print
# create files for categories and authors
for author in site_authors:
  filename = outdir + os.sep + 'db' + os.sep + "%s.%s" % ( slugify_name(author), fileext )
  
  print "@ Creating author file %s ..." % filename
  
  file = codecs.open( filename, "w+", "utf-8")
  file.write( """\
username: %s
avatar: put your avatar url here
email: put your email here
website: %s

put a description of yourself here""" % ( author, siteurl ) )
  file.close()

filename = outdir + os.sep + 'db' + os.sep + "categories.%s" % fileext
file     = codecs.open( filename, "w+", "utf-8")
print "@ Creating categories file %s ..." % filename
for category in site_categories:
  file.write( category + "\n" )
file.close()

print "\n@ Done :)"



# -*- coding: utf-8 -*-
# This file is part of SWG (Static Website Generator).
#
# Copyright(c) 2010-2011 Simone Margaritelli
# evilsocket@gmail.com
# http://www.evilsocket.net
# http://www.backbox.org
#
# This file may be licensed under the terms of of the
# GNU General Public License Version 2 (the ``GPL'').
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the GPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the GPL along with this
# program. If not, go to http://www.gnu.org/licenses/gpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
from mako.lookup     import TemplateLookup
from swg.core.config import Config

import swg

class TemplateManager:
  __instance = None
    
  def __init__(self):
    config = Config.getInstance()

    self.lookup = TemplateLookup( directories     = [config.tplpath],
                                  output_encoding = 'utf-8',
                                  input_encoding  = 'utf-8',
                                  encoding_errors = 'replace',
                                  cache_enabled   = True,
                                  cache_type      = 'file',
                                  cache_dir       = config.tplcache,
                                  collection_size = 1024
                                )

  def get( self, name ):
    return self.lookup.get_template(name)

  @classmethod
  def render( cls, template, **kwargs ):
    return template.render( config     = Config.getInstance(),
                            categories = swg.core.categorymanager.CategoryManager.getInstance().get(),
                            tags       = swg.core.tagmanager.TagManager.getInstance().get(),
                            **kwargs )

      

  @classmethod
  def getInstance(cls):
    if cls.__instance is None:
      cls.__instance = TemplateManager()
    return cls.__instance



# -*- coding: utf-8 -*-
# This file is part of SWG (Static Website Generator).
#
# Copyright(c) 2010-2011 Simone Margaritelli
# evilsocket@gmail.com
# http://www.evilsocket.net
# http://www.backbox.org
#
# This file may be licensed under the terms of of the
# GNU General Public License Version 2 (the ``GPL'').
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the GPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the GPL along with this
# program. If not, go to http://www.gnu.org/licenses/gpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
import datetime
import codecs
import os

class Config:
  __instance = None;

  version = '1.2.6'

  def __init__(self):
    self.now            = datetime.datetime.now()
    
    self.editor         = 'vim'
    self.datapath       = '.'
    self.dbpath         = os.path.join( self.datapath, 'db' )
    self.dbitem_ext     = "txt"
    self.hierarchy      = os.path.join( self.dbpath, 'categories.' + self.dbitem_ext )
    self.tplpath        = os.path.join( self.datapath, 'templates' )
    self.tplcache       = os.path.join( self.datapath, 'cache' )
    self.outputpath     = "output"
    self.copypaths      = {}

    self.siteurl        = ""
    self.sitename       = "Generated with SWG " + self.version
    self.charset        = "utf-8"
    self.language       = "en"
    self.keywords       = []
    
    self.basepath       = '/'
    self.page_ext       = "html"
    self.pager          = False
    self.items_per_page = 10

    self.gzip           = False
    self.compression    = 0
    self.tidyfy         = False
    
    self.transfer       = None

  def load( self, filename ):
    fd = codecs.open( filename, "r", "utf-8" )

    for line in iter(fd):
      line = line.strip()
      if line != '' and line[0] != '#':
        (key,value) = line.split( '=', 1 )
        key   = key.strip()
        value = value.strip()
        if key == 'dbitem_ext':
          self.dbitem_ext = value
        elif key == 'siteurl':
          self.siteurl = value 
        elif key == 'sitename':
          self.sitename = value
        elif key == 'charset':
          self.charset = value
        elif key == 'language':
          self.language = value
        elif key == 'basepath':
          self.basepath = value
        elif key == 'page_ext':
          self.page_ext = value
        elif key == 'editor':
          self.editor = value
        elif key == 'outputpath':
          self.outputpath = value
        elif key == 'pager':
          self.pager = True if value.lower() == 'true' else False
        elif key == 'gzip':
          self.gzip = True if value.lower() == 'true' else False
        elif key == 'tidyfy':
          self.tidyfy = True if value.lower() == 'true' else False
        elif key == 'compression':
          self.compression = int(value)
        elif key == 'items_per_page':
          self.items_per_page = int(value)
        elif key == 'copypaths':
          items = value.split(',')
          items = map( lambda s: s.strip(), items )
          for item in items:
            self.copypaths[ os.path.join( self.datapath, item ) ] = os.path.join( self.outputpath, item )
        elif key == 'keywords':
          items = value.split(',')
          self.keywords = map( lambda s: s.strip(), items )
        elif key == 'transfer':
          self.transfer = value
        else:
          raise Exception( "Unknown configuration key '%s'" % key )

    fd.close()

  @classmethod
  def getInstance(cls):
    if cls.__instance is None:
      cls.__instance = Config()
    return cls.__instance



# -*- coding: utf-8 -*-
# This file is part of SWG (Static Website Generator).
#
# Copyright(c) 2010-2011 Simone Margaritelli
# evilsocket@gmail.com
# http://www.evilsocket.net
# http://www.backbox.org
#
# This file may be licensed under the terms of of the
# GNU General Public License Version 2 (the ``GPL'').
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the GPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the GPL along with this
# program. If not, go to http://www.gnu.org/licenses/gpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
import math

from swg.core.config import Config

class Pager:
  def __init__( self, firstpage, format ):
    self.current   = 0
    self.pagen     = 0
    self.firstpage = firstpage
    self.format    = format
    self.pages     = []
    self.max       = 0
    self.left      = 0
    self.total     = 0
    self.config    = Config.getInstance()

  def setPages( self, pages ):
    self.pages = pages
    self.max   = len(pages)
    self.left  = self.max
    self.total = math.ceil( self.max / self.config.items_per_page )
    self.total += 1 if self.max % self.config.items_per_page != 0 else 0
    self.total = int(self.total)
    
  def getCurrentPageFilename(self):
    return self.format % self.pagen if self.pagen != 1 else self.firstpage

  def getCurrentPageNumber(self):
    return self.pagen

  def getTotalPages(self):
    return self.total

  def getCurrentPages(self):
    begin = (self.pagen - 1) * self.config.items_per_page
    end   = (begin + self.config.items_per_page) if self.left > self.config.items_per_page else (begin + self.left)
    return self.pages[begin:end]

  def goToNext(self):
    self.left = self.max - self.current
    if self.left <= 0:
      return False
    else:
      self.current += self.config.items_per_page if self.left > self.config.items_per_page else self.left
      return True

  def __iter__(self):
    return self

  def next(self):
    if self.goToNext() == False:
      raise StopIteration
    else:
      self.pagen += 1
      return self.getCurrentPageFilename()



from swg.core.config import Config
from setuptools import setup, find_packages
from distutils.util import convert_path
from fnmatch import fnmatchcase

import shutil
import os
import sys

def find_package_data( where               = '.', 
                       package             = '',
                       exclude             = ( '*.pyc', '*~', '*.bak', '*.swp*', '.*'),
                       exclude_directories = ('.*', 'CVS', '_darcs', './build', './dist', 'EGG-INFO', '*.egg-info'),
                       only_in_packages    = True,
                       show_ignored        = False):

  out = {}
  stack = [(convert_path(where), '', package, only_in_packages)]
  while stack:
    where, prefix, package, only_in_packages = stack.pop(0)
    for name in os.listdir(where):
      fn = os.path.join(where, name)
      if os.path.isdir(fn):
        bad_name = False
        for pattern in exclude_directories:
          if (fnmatchcase(name, pattern) or fn.lower() == pattern.lower()):
            bad_name = True
            if show_ignored:
                print >> sys.stderr, ("Directory %s ignored by pattern %s" % (fn, pattern))
            break
            if bad_name:
              continue
            if os.path.isfile(os.path.join(fn, '__init__.py')):
              if not package:
                  new_package = name
              else:
                  new_package = package + '.' + name
              stack.append((fn, '', new_package, False))
            else:
              stack.append((fn, prefix + name + '/', package, only_in_packages))
          elif package or not only_in_packages:
            # is a file
            bad_name = False
            for pattern in exclude:
              if (fnmatchcase(name, pattern) or fn.lower() == pattern.lower()):
                bad_name = True
                if show_ignored:
                  print >> sys.stderr, ( "File %s ignored by pattern %s" % (fn, pattern))
                break
            if bad_name:
              continue
            out.setdefault(package, []).append(prefix+name)

      return out

try:
  long_description = open( 'README.md', 'rt' ).read()
except:
  long_description = 'SWG - A static website generator'

setup( name                 = 'swg',
       version              = Config.version,
       description          = 'SWG - A static website generator',
       long_description     = long_description,
       author               = 'Simone Margaritelli',
       author_email         = 'evilsocket@gmail.com',
       url                  = 'http://www.github.com/evilsocket/swg',
       packages             = find_packages(),
       include_package_data = True,
       package_data         = find_package_data( package = 'swg', only_in_packages = False ),
       install_requires     = ( 'mako >= 0.4.1', 'pytidylib >= 0.2.1' ),
       scripts              = [ 'swg/swg' ],
       license              = 'GPL',
       zip_safe             = False,
       classifiers          = [
            'Development Status :: 4 - Beta',
            'Environment :: Console',
            'Intended Audience :: End Users/Desktop',
            'Intended Audience :: Developers',
            'Intended Audience :: System Administrators',
            'License :: OSI Approved :: GNU General Public License (GPL)',
            'Operating System :: MacOS :: MacOS X',
            'Operating System :: Unix',
            'Operating System :: POSIX',
            'Operating System :: Microsoft :: Windows',
            'Programming Language :: Python',
            'Topic :: Software Development',
            'Topic :: Software Development :: Build Tools',
            'Topic :: Software Development :: Code Generators',
            'Topic :: Internet',
            'Topic :: Internet :: WWW/HTTP :: Site Management',
      ]
)




# -*- coding: utf-8 -*-
# This file is part of SWG (Static Website Generator).
#
# Copyright(c) 2010-2011 Simone Margaritelli
# evilsocket@gmail.com
# http://www.evilsocket.net
# http://www.backbox.org
#
# This file may be licensed under the terms of of the
# GNU General Public License Version 2 (the ``GPL'').
#
# Software distributed under the License is distributed
# on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
# express or implied. See the GPL for the specific language
# governing rights and limitations.
#
# You should have received a copy of the GPL along with this
# program. If not, go to http://www.gnu.org/licenses/gpl.html
# or write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
import os

from swg.core.authorparser import AuthorParser
from swg.core.config       import Config

class AuthorManager:
  __instance = None

  def __init__(self):
    self.authors = {}

  def get( self, username ):
    id = username.lower()
    if not self.authors.has_key(id):
      self.authors[id] = AuthorParser().parse( os.path.join( Config.getInstance().dbpath, ( "%s.%s" % (username,Config.getInstance().dbitem_ext) ) ) )

    return self.authors[id]

  @classmethod
  def getInstance(cls):
    if cls.__instance is None:
      cls.__instance = AuthorManager()
    return cls.__instance
